/*
 *   Copyright 2011-2012 Giuseppe Calà <gcala@mail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "browsefirefoxbookmarksrunner.h"

#include <KDebug>
#include <KDirWatch>
#include <KToolInvocation>

#include <QDir>
#include <QFile>

#include <QSqlError>
#include <QSqlQuery>

BrowseFirefoxBookmarksRunner::BrowseFirefoxBookmarksRunner(QObject *parent, const QVariantList& args)
        : Plasma::AbstractRunner(parent, args)
{
    Q_UNUSED(args);

	connection_ = QSqlDatabase::addDatabase("QSQLITE", CONNECTIONNAME);

    setObjectName(QLatin1String( "Browse Firefox Bookmarks Runner" ));
    setSpeed(AbstractRunner::SlowSpeed);
	m_icon = KIcon(QLatin1String( "firefox-bookmarks" ));

	reloadConfiguration();
	if(!loadBookmarks())
		return;

	// listen for changes to the list of recent documents
	KDirWatch *historyWatch = new KDirWatch(this);
	historyWatch->addFile(m_profilesPath.absolutePath() + QDir::separator() + m_profileFolder + QDir::separator() + "places.sqlite");
	connect(historyWatch,SIGNAL(dirty(QString)),this,SLOT(loadBookmarks()));
	connect(historyWatch,SIGNAL(created(QString)),this,SLOT(loadBookmarks()));
	connect(historyWatch,SIGNAL(deleted(QString)),this,SLOT(loadBookmarks()));

	Plasma::RunnerSyntax s(i18nc("Firefox bookmarks runner syntax, first word is trigger word, e.g.  \"bfb\".","%1:q:", m_triggerWord),i18n("Search Firefox bookmarks of :q:."));

	if (!m_requireTriggerWord) {
        s.addExampleQuery(QLatin1String( ":q:" ));
    }

    QList<Plasma::RunnerSyntax> syns;
    syns << s;
    setSyntaxes(syns);
}

BrowseFirefoxBookmarksRunner::~BrowseFirefoxBookmarksRunner()
{
    connection_.close();
    QSqlDatabase::removeDatabase(CONNECTIONNAME);
}

bool BrowseFirefoxBookmarksRunner::loadBookmarks()
{
	m_names.clear();
	m_urls.clear();

	QStringList ids(m_enabledBookmarksFolders.split(",", QString::SkipEmptyParts));
	QString whereClause;
	foreach(QString id, ids) {
		whereClause += "moz_bookmarks.parent=" + id + " OR ";
	}
	whereClause.replace(QRegExp("\\sOR\\s$"), "");

	QSqlQuery query("SELECT moz_bookmarks.title,moz_places.url "
	"FROM moz_bookmarks "
	"LEFT JOIN moz_places ON moz_bookmarks.fk=moz_places.id "
	"WHERE (" + whereClause + ") AND moz_places.url IS NOT NULL AND moz_places.url NOT LIKE '%place:%'",connection_);

	while (query.next()) {
        m_names << query.value(0).toString();
        m_urls << query.value(1).toString();
    }

	return true;
}

bool BrowseFirefoxBookmarksRunner::selectDefaultProfile()
{
	bool newProfile = false;
	bool firstProfile = true;
	bool isDefault = false;
	QString name;
	QString path;

	QFile file(m_profilesPath.absolutePath() + QDir::separator() + "profiles.ini");

	if(!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
		qDebug() << "can't open " << file.fileName();
		return false;
	}

	while (!file.atEnd()) {
        QString line = file.readLine();
        line.trimmed();
        line.replace(QRegExp("\\n$"),"");

		if(line.startsWith("[Profile")) {
            newProfile = true;
            continue;
        }

        if(newProfile) {
            if(line.contains("Name=")) {
                name = line.split("Name=").at(1);
            }
            else if(line.contains("Path=")) {
                path = line.split("Path=").at(1);
            }
            else if(line.contains("Default=1")) {
				isDefault = true;
			}
            else if(line.isEmpty()) {
                newProfile = false;
            }
        }

        if(!newProfile && !name.isEmpty()) {
			if(firstProfile || isDefault) {
				m_profileName = name;
				m_profileFolder = path;
				firstProfile = false;
				isDefault = false;
			}
        }
	}

	file.close();

	return true;
}

void BrowseFirefoxBookmarksRunner::match(Plasma::RunnerContext &context)
{
	QString term = context.query();

	if (term.length() < 2) {
		return;
	}

	if (m_requireTriggerWord) {
		if (!term.startsWith(m_triggerWord)) //ignore queries without the triggerword
		{
			return;
		}
	}

	term.replace(QRegExp("^" + m_triggerWord), "").trimmed(); //remove the triggerword

	int size = m_names.size();

	for(int i = 0; i < size; i++) {
// 		if (!context.isValid()) {
// 			return;
// 		}

		if ( m_names.at(i).contains(term, Qt::CaseInsensitive) || m_urls.at(i).contains(term, Qt::CaseInsensitive) ) {
			Plasma::QueryMatch match(this);
			match.setType(Plasma::QueryMatch::PossibleMatch);
			match.setRelevance(0.4);
			match.setIcon(m_icon);
			match.setData(m_urls.at(i));
			match.setText(m_names.at(i));
			context.addMatch(term, match);
		}
	}
}

void BrowseFirefoxBookmarksRunner::run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match)
{
	Q_UNUSED(context)
	const QString location = match.data().toString();
	kDebug() << "Browse to " << location;
	if (!location.isEmpty()) {
		KToolInvocation::invokeBrowser(location);
	}
}

void BrowseFirefoxBookmarksRunner::reloadConfiguration()
{
    KSharedConfig::Ptr cfg = KSharedConfig::openConfig(QLatin1String("krunnerrc"));
    KConfigGroup grp = cfg->group("Runners");
    grp = KConfigGroup(&grp, "Browse Firefox Bookmarks Runner");

    m_profilesPath.setPath(grp.readEntry("ProfilesPath", QDir::homePath() + QDir::separator() + ".mozilla/firefox/"));
    m_profileName = grp.readEntry("Name", "");
    m_profileFolder = grp.readEntry("Folder", "");
    m_enabledBookmarksFolders = grp.readEntry("EnabledBookmarksFolders", "2");

    m_triggerWord = grp.readEntry("trigger", i18n("bfb"));
    //Processing will be triggered by "keyword "
    m_triggerWord += QLatin1Char( ' ' );

    m_requireTriggerWord = grp.readEntry("requireTriggerWord", true);

    createConnection();
    loadBookmarks();
}

bool BrowseFirefoxBookmarksRunner::createConnection()
{
    if(connection_.isOpen())
        connection_.close();

    connection_.setDatabaseName(m_profilesPath.absolutePath() + QDir::separator() + m_profileFolder + "/places.sqlite");

    if (!connection_.open()) {
        qDebug() << "can't open " << m_profilesPath.absolutePath() + QDir::separator() + m_profileFolder + "/places.sqlite";
        qDebug() << connection_.lastError().text();
        return false;
    }

    return true;
}

#include "browsefirefoxbookmarksrunner.moc"
