/*
 *   Copyright 2011-2012 Giuseppe Calà <gcala@mail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

//Project-Includes
#include "browsefirefoxbookmarksrunner_config.h"

//KDE-Includes
#include <Plasma/AbstractRunner>
#include <KDE/KFileDialog>
#include <KDE/KMessageBox>
#include <KDE/KAboutData>
#include <QDebug>
#include <QSqlError>
#include <QSqlQuery>


K_EXPORT_RUNNER_CONFIG(browsefirefoxbookmarks, BrowseFirefoxBookmarksRunnerConfig)

BrowseFirefoxBookmarksRunnerConfigForm::BrowseFirefoxBookmarksRunnerConfigForm(QWidget* parent) : QWidget(parent)
{
    setupUi(this);
}

BrowseFirefoxBookmarksRunnerConfig::BrowseFirefoxBookmarksRunnerConfig(QWidget* parent, const QVariantList& args) :
        KCModule(ConfigFactory::componentData(), parent, args)
{
	KAboutData *about = new KAboutData(
		"browsefirefoxbookmarksrunner", 0, ki18n( "Browse Firefox Bookmarks Runner" ),
		"0.2.4", ki18n("Searches in Firefox's bookmarks"), KAboutData::License_GPL,
		ki18n( "Copyright 2011-2012 Giuseppe Calà" ) );
		about->addAuthor( ki18n("Giuseppe Calà") );
		setAboutData( about );

	connection_ = QSqlDatabase::addDatabase( "QSQLITE", CONNECTIONNAME );

    m_ui = new BrowseFirefoxBookmarksRunnerConfigForm(this);
	m_ui->changeProfilesPathButton->setText(i18n("Change Profiles File"));
	m_ui->changeProfilesPathButton->setIcon(KIcon("firefox-bookmarks"));
	m_ui->changeProfilesPathButton->setMinimumHeight(26);

	m_ui->infoLabel->setText(i18n("Select bookmarks' folders:"));

    QGridLayout* layout = new QGridLayout(this);
    layout->addWidget(m_ui, 0, 0);

	connect(m_ui->profilesComboBox, SIGNAL(activated(int)), this, SLOT(profileSelected()));
	connect(m_ui->changeProfilesPathButton, SIGNAL(clicked()), this, SLOT(selectProfilesFolder()));
	connect(m_ui->foldersListWidget, SIGNAL(itemClicked ( QListWidgetItem * )), this, SLOT(bookmarksFolderSelected()));
	connect(m_ui->requireTriggerWord, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->requireTriggerWord, SIGNAL(stateChanged(int)), this, SLOT(toggleTriggerWord(int)));
    connect(m_ui->triggerWord, SIGNAL(textChanged(const QString&)), this, SLOT(changed()));

	load();
	if(!createConnection())
		return;
}

BrowseFirefoxBookmarksRunnerConfig::~BrowseFirefoxBookmarksRunnerConfig()
{
//	QSqlDatabase db = QSqlDatabase::database();
    if(connection_.isOpen())
        connection_.close();
	QSqlDatabase::removeDatabase(CONNECTIONNAME);
}

void BrowseFirefoxBookmarksRunnerConfig::load()
{
	KCModule::load();

	KSharedConfig::Ptr cfg = KSharedConfig::openConfig(QLatin1String("krunnerrc"));
	KConfigGroup grp = cfg->group("Runners");
	grp = KConfigGroup(&grp, "Browse Firefox Bookmarks Runner");

	m_profilesPath.setPath(grp.readEntry("ProfilesPath", QDir::homePath() + QDir::separator() + ".mozilla/firefox/"));
	m_profileName = grp.readEntry("Name", "");
	m_profileFolder = grp.readEntry("Folder", "");
	m_enabledBookmarksFolders = grp.readEntry("EnabledBookmarksFolders", "2");

	const bool requireTrigger = grp.readEntry("requireTriggerWord", true);
    const QString trigger = grp.readEntry("trigger", i18n("bfb"));

    if (!requireTrigger) {
        m_ui->triggerWord->setEnabled(false);
    }

    m_ui->requireTriggerWord->setCheckState( (requireTrigger) ? Qt::Checked : Qt::Unchecked );
    m_ui->triggerWord->setText( trigger );

	loadProfiles();
	populateFoldersListWidget();

	emit changed(false);
}

void BrowseFirefoxBookmarksRunnerConfig::loadProfiles()
{
	profiles.clear();

	m_ui->profilesComboBox->clear();

	bool newProfile = false;
	QString name;
	QString path;
	bool isDefault = false;
	bool foundDefault = false;

	QFile file(m_profilesPath.absolutePath() + QDir::separator() + "profiles.ini");

	if(!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
		qDebug() << "can't open " << file.fileName();
		return;
	}

	while (!file.atEnd()) {
        QString line = file.readLine();
        line.trimmed();
        line.replace(QRegExp("\\n$"),"");

		if(line.startsWith("[Profile")) {
            newProfile = true;
            continue;
        }

        if(newProfile) {
            if(line.contains("Name=")) {
                name = line.split("Name=").at(1);
            }
            else if(line.contains("Path=")) {
                path = line.split("Path=").at(1);
            }
            else if(line.contains("Default=1")) {
				isDefault = true;
				foundDefault = true;
			}
            else if(line.isEmpty()) {
                newProfile = false;
            }
        }

        if(!newProfile && !name.isEmpty()) {
			Profile p;
			p.name = name;
			p.path = path;
			p.isDefault = isDefault;

			profiles.append(p);
			isDefault = false;
			name = "";
			path = "";
        }
	}

	file.close();

	if(!foundDefault)
		profiles[0].isDefault = true;

	foreach(Profile p, profiles) {
		if(p.isDefault)
			m_ui->profilesComboBox->addItem(p.name + i18n(" [Default]"), p.path);
		else
			m_ui->profilesComboBox->addItem(p.name, p.path);
	}

	if(!m_profileName.isEmpty() && !m_profileFolder.isEmpty()) {
		for(int i = 0; i < m_ui->profilesComboBox->count(); i++) {
			if(m_ui->profilesComboBox->itemText(i).replace(i18n(" [Default]"), "") == m_profileName) {
				m_ui->profilesComboBox->setCurrentIndex(i);
				profiles[m_ui->profilesComboBox->currentIndex()].bookmarksFolders = m_enabledBookmarksFolders;
			}
		}
		profileSelected();
	}
}

void BrowseFirefoxBookmarksRunnerConfig::save()
{
	KCModule::save();

	KSharedConfig::Ptr cfg = KSharedConfig::openConfig(QLatin1String("krunnerrc"));
	KConfigGroup grp = cfg->group("Runners");
	grp = KConfigGroup(&grp, "Browse Firefox Bookmarks Runner");

	grp.writeEntry("ProfilesPath", m_profilesPath.absolutePath());
	grp.writeEntry("Name", m_ui->profilesComboBox->currentText().replace(i18n(" [Default]"), ""));
	grp.writeEntry("Folder", m_ui->profilesComboBox->itemData(m_ui->profilesComboBox->currentIndex()).toString());

	grp.writeEntry("EnabledBookmarksFolders", profiles.at(m_ui->profilesComboBox->currentIndex()).bookmarksFolders);

	bool requireTrigger = m_ui->requireTriggerWord->checkState() == Qt::Checked;
    if (requireTrigger) {
        grp.writeEntry( "trigger", m_ui->triggerWord->text() );
    }
    grp.writeEntry( "requireTriggerWord", requireTrigger );

	grp.sync();

	emit changed(false);
}

void BrowseFirefoxBookmarksRunnerConfig::defaults()
{
	m_ui->requireTriggerWord->setCheckState( Qt::Checked );
    m_ui->triggerWord->setText( i18n("bfb") );
    emit changed(true);
}

void BrowseFirefoxBookmarksRunnerConfig::selectProfilesFolder()
{
	QFileInfo profilesFile(KFileDialog::getOpenFileName(QDir::homePath(), "profiles.ini", 0, i18n("Select a custom profile file")));

	if(!profilesFile.exists()) {
		KMessageBox::error(this, i18n("You must select a valid Firefox profiles.ini file."), i18n("Error"));
	}
	else {
		m_ui->profilesComboBox->clear();
		m_profilesPath.setPath(profilesFile.absolutePath());
		m_profileName = "";
		m_profileFolder = "";
		loadProfiles();
	}
}

void BrowseFirefoxBookmarksRunnerConfig::profileSelected()
{
	m_ui->foldersListWidget->clear();
	for(int i = m_ui->foldersListWidget->count() -1; i >=0; i--) {
		QListWidgetItem *item = m_ui->foldersListWidget->takeItem(i);
		delete item;
	}

	if(!m_ui->profilesComboBox->currentText().contains(i18n("[Default]")))
		m_ui->messageLabel->setText(i18n("You are using a profile that is not default in Firefox."));
	else
		m_ui->messageLabel->setText(i18n(""));

	//NOTE:
	createConnection();
	populateFoldersListWidget();
	// selezionare quella di default

	changed();
}

bool BrowseFirefoxBookmarksRunnerConfig::createConnection()
{
//    QSqlDatabase db = QSqlDatabase::database();
    if(connection_.isOpen())
        connection_.close();

    connection_.setDatabaseName(m_profilesPath.absolutePath() + QDir::separator() + profiles.at(m_ui->profilesComboBox->currentIndex()).path + "/places.sqlite");

    if (!connection_.open()) {
        qDebug() << "can't open " << m_profilesPath.absolutePath() + QDir::separator() + m_profileFolder + "/places.sqlite";
		qDebug() << connection_.lastError().text();
        return false;
    }

    return true;
}

void BrowseFirefoxBookmarksRunnerConfig::populateFoldersListWidget()
{
	m_ui->foldersListWidget->clear();

	QStringList currentFoldersIds(profiles.at(m_ui->profilesComboBox->currentIndex()).bookmarksFolders.split(",", QString::SkipEmptyParts));

	for(int i = m_ui->foldersListWidget->count() -1; i >=0; i--) {
		QListWidgetItem *item = m_ui->foldersListWidget->takeItem(i);
		delete item;
	}

	QStringList ids;
	QSqlQuery query("SELECT DISTINCT( "
					"SELECT id "
					"FROM moz_bookmarks as other "
					"WHERE id = moz_bookmarks.parent "
					")  as id "
					"FROM moz_bookmarks, moz_places "
					"WHERE moz_bookmarks.fk = moz_places.id and moz_places.url not like '%place:%' "
					"ORDER by id", connection_);

	while (query.next()) {
         ids <<  query.value(0).toString();
    }

    foreach(QString id, ids){
		QListWidgetItem *item = new QListWidgetItem(getFolderName(id));
		item->setData(Qt::UserRole, id);
		if(currentFoldersIds.contains(id))
			item->setCheckState(Qt::Checked);
		else
			item->setCheckState(Qt::Unchecked);
        m_ui->foldersListWidget->addItem(item);
	}
}

QString BrowseFirefoxBookmarksRunnerConfig::getFolderName(const QString &id)
{
	QString name;
	QSqlQuery query("SELECT DISTINCT title FROM moz_bookmarks WHERE id=" + id, connection_);

	while (query.next()) {
         name =  query.value(0).toString();
    }

	return name;
}

void BrowseFirefoxBookmarksRunnerConfig::bookmarksFolderSelected()
{
	QString folderIds;
	for (int i = 0; i < m_ui->foldersListWidget->count(); i++) {
        if (m_ui->foldersListWidget->item(i)->checkState() == Qt::Checked) {
            folderIds += "," + m_ui->foldersListWidget->item(i)->data(Qt::UserRole).toString();
        }
	}

	profiles[m_ui->profilesComboBox->currentIndex()].bookmarksFolders = folderIds;

	changed();
}

void BrowseFirefoxBookmarksRunnerConfig::toggleTriggerWord(int state)
{
    if (state == Qt::Unchecked) {
        m_ui->triggerWord->setEnabled(false);
    } else {
        m_ui->triggerWord->setEnabled(true);
    }
}
